/**
 * Create a dialog containing functions and routines to crop an image.
 *
 * The .presentationUiImageCrop() creates a dialog that contains different possibilities to crop/manipulate an image, e.g. crop, zoom, rotate.
 *
 * ### Examples
 * Binds the presentationUiImageCrop()-Dialog to a button. If clicked the dialog appears and the image can be cropped/manipulated.
 *
 * HTML:
 * <div class="ep-button ep-button-cta ep-button-imagecrop"></div>
 *
 * JavaScript:
 *
 *    <script>
 *        require([
 *           'jquery',
 *           'ep',
 *           'de_epages/presentation/ui/imagecrop',
 *
 *           '$ready!'
 *       ], function($, ep, de_epages) {
 *            $('#ep-button-imagecrop').presentationUiImageCrop({
 *              img: $('<img src="https://i.vimeocdn.com/video/753086196.jpg" />')
 *            });
 *       });
 *   </script>
 *
 *
 *
 * @class jQuery.ui.presentationUiImageCrop
 * @extends jQuery.widget
 *
 * @uses ep
 * @uses de_epages
 * @uses $tmpl!./imagecrop
 * @uses jquery/cropperjs/cropper
 * @uses ep.dict
 * @uses jQuery.ui.widget
 * @uses ep.ajax
 * @uses ep.ui.dialog
 * @uses jquery/tmpl
 * @since 7.39.0
 */

/**
 * @cfg {jQueryObject} [img] jQuery object representing the image that should be cropped.
 */

/**
 * @cfg {String} [id] ID of the image.
 */

/**
 * @cfg {Function} [onAfterCrop] Function/Callback executed after the cropping and resetting the ui.
 */

/**
 * @cfg {Function} [onInterruptSave] Function/Callback executed instead of saving the crop data via AJAX request.
 */

/**
 * @cfg {String} [defaultAspectRatio] String of the default aspect ratio (possible values: '1:1',  '2:3', '4:3', '16:9', '3:2', default: '3:2').
 */

/**
 * @cfg {Object} [dialog] Object to dimension the dialog.
 */

/**
 * @cfg {Number} [objectID] ID of the object the cropped image belongs to.
 */

/**
 * @cfg {Boolean} [resize] Flag to decide if an image should be resized during initialize.
 */

/**
 * @cfg {Number} [zoomRatio] Ratio for zoom if a mousewheel is used.
 */

/**
 * @cfg {Boolean} [useDisableCrop] If set to true a button to disable cropping is displayed.
 */

/**
 * See `jQuery.ui.presentationUiImageCrop` for details.
 *
 * @param {jQueryObject} [img] jQuery object representing the image that should be cropped.
 * @param {String} [id] ID of the image.
 * @param {Function} [onAfterCrop] Function/Callback executed after the cropping and resetting the ui.
 * @param {Function} [onInterruptSave] Function/Callback executed instead of saving the crop data via AJAX request.
 * @param {String} [defaultAspectRatio] String of the default aspect ratio (possible values: '1:1',  '2:3', '4:3', '16:9', '3:2', default: '3:2').
 * @param {Object} [dialog] Object to dimension the dialog.
 * @param {Number} [objectID] ID of the object the cropped image belongs to.
 * @param {Boolean} [resize] Flag to decide if an image should be resized during initialize.
 * @param {Number} [zoomRatio] Ratio for zoom if a mousewheel is used.
 * @param {Boolean} [useDisableCrop] If set to true a button to disable cropping is displayed.
 *
 *
 * @method presentationUiImageCrop
 * @member jQuery
 *
 * @since 7.39.0
 */

/*
 * @copyright       © Copyright 2006-2018, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages/presentation/ui/imagecrop
 */

define("de_epages/presentation/ui/imagecrop", [
    "jquery",
    "ep",
    "de_epages",
    "$tmpl!./imagecrop",
    "$dict!../dictionary",

    "jquery/cropperjs/cropper",
    "jquery/ui/widget",
    "ep/ajax",
    "ep/ui/dialog",
    "jquery/tmpl"
], function($, ep, de_epages, tmplImageCropDialog, presentationDict) {
/*
 * @dictionary      ep.dict
 *
 * @translation     {Apply}
 *                  {Cancel}
 *                  {Height}
 *                  {MoveLeft}
 *                  {MoveRight}
 *                  {MoveTop}
 *                  {MoveDown}
 *                  {Reset}
 *                  {Save}
 *                  {Width}
 *                  {ZoomIn}
 *                  {ZoomOut}
 *
 * @dictionary      de_epages.presentation.dictionary
 *
 * @translation     {DisableCrop}
 *                  {UserDefined}

 */

    // translation object
    var translations = {
            Apply: presentationDict.translate('Apply'),
            Cancel: presentationDict.translate('Cancel'),
            Height: presentationDict.translate('Height'),
            MoveLeft: presentationDict.translate('MoveLeft'),
            MoveRight: presentationDict.translate('MoveRight'),
            MoveUp: presentationDict.translate('MoveTop'),
            MoveDown: presentationDict.translate('MoveDown'),
            Reset: presentationDict.translate('Reset'),
            Save: presentationDict.translate('Save'),
            Width: presentationDict.translate('Width'),
            ZoomIn: presentationDict.translate('ZoomIn'),
            ZoomOut: presentationDict.translate('ZoomOut'),
            DisableCrop: presentationDict.translate('DisableCrop'),
            UserDefined: presentationDict.translate('UserDefined')
        },

        getSanitizedCropSizeValue = function(inputValue) {
            var numberValue = Number(inputValue.replace(',', '.')),
                value = numberValue ? Math.round(numberValue) : inputValue.replace(/[^0-9]/g, '');

            return Number(value);
        };

    $.widget('ui.presentationUiImageCrop', {
    //  dialog,
    //  src,
    //  currentAspectRatio,
    //  hTimer,

        tmplElements: {},

        buttonActiveClass: "ep-imagecrop-toolbar-button-active",
        customAspectRatioDiabledClass: "ep-imagecrop-toolbar-aspectratio-custom-group-disabled",

        inputDelay: 600,
        aspectRatioCustom: false,
        aspectRatioLock: true,
        aspectRatioMap: {
            '1:1': {
                args: [1 / 1], method: 'setAspectRatio', class: 'ep-icon-cropping-1-1', label: '1:1'
            },
            '2:3': {
                args: [2 / 3], method: 'setAspectRatio', class: 'ep-icon-cropping-2-3', label: '2:3'
            },
            '4:3': {
                args: [4 / 3], method: 'setAspectRatio', class: 'ep-icon-cropping-4-3', label: '4:3'
            },
            '16:9': {
                args: [16 / 9], method: 'setAspectRatio', class: 'ep-icon-cropping-16-9', label: '16:9'
            },
            '3:2': {
                args: [3 / 2], method: 'setAspectRatio', class: 'ep-icon-cropping-3-2', label: '3:2'
            }
        },

        rotateMap: {
            'acw-90': {
                args: [-90], method: 'rotate', class: 'fa fa-undo', label: '-90°'
            },
            'cw-90': {
                args: [90], method: 'rotate', class: 'fa fa-repeat', label: '90°'
            }
        },

        navigationButtonMap: {
            'zoom-plus': {
                args: [0.1], method: 'zoom', class: 'fa fa-search-plus', label: translations.ZoomIn
            },
            'zoom-minus': {
                args: [-0.1], method: 'zoom', class: 'fa fa-search-minus', label: translations.ZoomOut
            },
            'move-left': {
                args: [-1, 0], method: 'move', class: 'fa fa-arrow-left', label: translations.MoveLeft
            },
            'move-right': {
                args: [1, 0], method: 'move', class: 'fa fa-arrow-right', label: translations.MoveRight
            },
            'move-up': {
                args: [0, -1], method: 'move', class: 'fa fa-arrow-up', label: translations.MoveUp
            },
            'move-down': {
                args: [0, 1], method: 'move', class: 'fa fa-arrow-down', label: translations.MoveDown
            },
            'reset': {
                args: [0], method: 'reset', class: "fa fa-refresh", label: translations.Reset
            }
        },

        options: {
        //  img,
        //  id,
        //  onAfterCrop,
        //  onInterruptSave,

            objectID: 1,
            resize: true,
            resizeDim: 500,
            variation: false,

            zoomRatio: 0.1,
            defaultAspectRatio: '3:2',
            useDisableCrop: true,

            dialog: {
                width: 766,
                height: 'auto',
                cta: 'Save'
            }
        },

        /**
         * Method is called during initialization, when widget is created.
         *
         * @member jQuery.ui.presentationUiImageCrop
         *
         * @since 7.39.0
         */
        _create: function() {
            var self = this,
                o = self.options,
                elem = self.element,
                name;

            if (o.img !== undefined && ep.isJQueryElement(o.img) && o.id !== undefined) {
                // only the first image
                o.img = o.img.eq(0);

                // set name data attribute if necessary
                if (!o.img.data('name')) {
                    name = o.img[0].src.split('/');
                    o.img.data('name', name[name.length - 1]);
                }

                self._resizeImage();
            } else {
                return ep;
            }

            elem.on('click.imageCrop', function(evt) {
                evt.preventDefault();
                evt.stopPropagation();

                self.open();
            });
        },

        /**
         * Resize image to m size if necessary. Do not resize for new product and variation product images.
         *
         * @member jQuery.ui.presentationUiImageCrop
         *
         * @since 7.39.0
         */
        _resizeImage: function() {
            var self = this,
                o = self.options,
                tmpImg,
                naturalWidth,
                naturalHeight,
                canvas,
                canvasHeight,
                canvasWidth,
                ctx;

            // get natural dimensions
            naturalWidth = o.img[0].naturalWidth;
            naturalHeight = o.img[0].naturalHeight;

            // create temporary image to get natural dimensions
            tmpImg = new Image();
            tmpImg.src = o.img[0].src;

            //  resize image if resize needed AND natural dimensions exceed maximal allowed dimensions
            if (o.resize && Math.max(naturalWidth, naturalHeight) > o.resizeDim) {
                // get new dimensions
                if (naturalWidth >= naturalHeight) {
                    canvasWidth = o.resizeDim;
                    canvasHeight = (naturalHeight / naturalWidth) * o.resizeDim;
                } else {
                    canvasHeight = o.resizeDim;
                    canvasWidth = (naturalWidth / naturalHeight) * o.resizeDim;
                }

                // create canvas
                canvas = $('<canvas width="' + parseInt(canvasWidth) + '" height="' + parseInt(canvasHeight) + '" />');
                // get context
                ctx = canvas[0].getContext('2d');
                // scale image
                ctx.drawImage(o.img[0], 0, 0, canvasWidth, canvasHeight);

                self.src = canvas[0].toDataURL();
            } else {
                self.src = o.img.attr('src');
            }
        },

        /**
         * Method renders the slider template and creates the dialog.
         *
         * @member jQuery.ui.presentationUiImageCrop
         *
         * @since 7.39.0
         */
        _createDialog: function() {
            var self = this,
                o = self.options,
                elem = self.element,
                renderedTmpl;

            self.currentAspectRatio = self.currentAspectRatio ? self.currentAspectRatio : o.defaultAspectRatio

            renderedTmpl = tmplImageCropDialog([{
                currentAspectRatio: self.currentAspectRatio,
                aspectRatioMap: self.aspectRatioMap,
                rotateMap: self.rotateMap,
                useDisableCrop: o.useDisableCrop,
                navigationButtonMap: self.navigationButtonMap,
                src: self.src,
                dict: translations
            }]);

            // get DOM-Elements from template.
            $.extend(self.tmplElements, renderedTmpl.tmplItem('elements'));
            // initialize dialog widget
            self.dialog = $('<div/>').html(renderedTmpl).uiDialog({
                width: o.dialog.width,
                height: o.dialog.height,
                modal: true,
                classes: {
                  'ui-dialog': 'ep-imagecrop-dialog'
                },
                position: {
                    my: "center top",
                    at: "center top+" + (30 + window.pageYOffset).toString(),
                    of: $('body')
                },
                buttons: [
                    {
                        text: translations[o.dialog.cta] ? translations[o.dialog.cta] : translations.Save,
                        click: function() {
                            self._handleCropData();
                            self.close();
                        },
                        cssClass: 'ep-button ep-button-cta'
                    },
                    {
                        text: translations.Cancel,
                        click: function () {
                            self._destroyData();
                            self.close();
                        },
                        cssClass: 'ep-button ep-button-close'
                    }
                ],

                create: function(event, ui) {
                    // handle cancel button in the titlebar
                    $(this).closest('div.ui-dialog').find('.ui-dialog-titlebar-close').on('click', function() {
                        self._destroyData();
                    });
                },

                open: function(event, ui) {
                    var dialogWidget = $(this).data('uiUiDialog'),
                        $this = $(this);

                    // move overlay before this dialog node
                    window.setTimeout(function() {
                        dialogWidget.overlay.removeAttr("style");
                        $this.closest('div.ui-dialog').before(dialogWidget.overlay);
                    }, 30);
                }
            });

            // initialize the cropper plugin
            self._initImageCrop({});


            // handle aspect ratio button clicks
            self.tmplElements.aspectRatio.on('click.imageCrop', function() {
                var $this = $(this);

                self._handleAspectRatioButton({
                    aspectRatioCurrent: $this.data('aspectRatioCurrent'),
                    button: $this,
                    method: $this.data('method'),
                    args: $this.data('args')
                });
            });

            // handle custom aspect ratio button click
            self.tmplElements.aspectRatioCustom.on('click.imageCrop', function() {
                self._enableAspectRatioCustom();
            });

            // handle aspect ratio lock/unlock button click
            self.tmplElements.aspectRatioLock.on('click.imageCrop', function() {
                self.aspectRatioCustom && self._handleAspectRatioLock();
            });

            // handle crop box height in custom aspect ratio mode
            self.tmplElements.cropHeight.on('input.imageCrop', function() {
                if (self.hTimer) {
                    window.clearTimeout(self.hTimer);
                    self.hTimer = undefined;
                }

                self.hTimer = window.setTimeout(function() {
                    self._handleCropHeight();
                }, self.inputDelay);
            });

            // handle crop box width in custom aspect ratio mode
            self.tmplElements.cropWidth.on('input.imageCrop', function() {
                if (self.hTimer) {
                    window.clearTimeout(self.hTimer);
                    self.hTimer = undefined;
                }

                self.hTimer = window.setTimeout(function() {
                    self._handleCropWidth();
                }, self.inputDelay);
            });

            // handle custom aspect ratio button click
            self.tmplElements.disableCrop && self.tmplElements.disableCrop.on('click.imageCrop', function() {
                // handle disable crop button
                var $this = $(this);

                self._handleDisableCrop({
                    button: $this,
                    method: $this.data('method'),
                    args: $this.data('args')
                });
            });

            // handle toolbar button clicks
            self.tmplElements.navigationButton.on('click.imageCrop', function() {
                var $this = $(this);

                self._handleButton({
                    method: $this.data('method'),
                    args: $this.data('args')
                });
            });

            // handle toolbar button clicks
            self.tmplElements.rotateButton.on('click.imageCrop', function() {
                var $this = $(this);

                self._handleButton({
                    method: $this.data('method'),
                    args: $this.data('args')
                });
            });
        },

        /**
         * Resets dialog, image source and cropper widget.
         *
         * @member jQuery.ui.presentationUiImageCrop
         *
         * @since 7.39.0
         */
        _resetUI: function() {
            var self = this,
                o = self.options,
                dataUrl = self.tmplElements.image.cropper('getCroppedCanvas').toDataURL(),
                cropData = self._getCropData(),
                newImg = o.img.clone();

            newImg.attr('src', dataUrl);
            newImg.data('cropdata', JSON.stringify(cropData));
            newImg.data('name', cropData.name);

            // destroy widgets and reset data
            self._destroyData();

            // set new source
            self.src = dataUrl;

            // replace original image with cropped image
            o.img.replaceWith(newImg);
            o.img = newImg;

            if (o.onAfterCrop && typeof o.onAfterCrop === 'function') {
                o.onAfterCrop({
                    dataUrl: dataUrl,
                    cropData: cropData
                });
            }
        },

        /**
         * Method destroys additional widgets and reset widget properties.
         *
         * @member jQuery.ui.presentationUiImageCrop
         *
         * @since 7.39.0
         */
        _destroyData: function() {
            var self = this,
                o = self.options,
                cropData = self.tmplElements.image.cropper('getData');

            // reset cropper
            self.tmplElements.image.cropper('reset');

            // destroy widgets
            self.tmplElements.image.cropper('destroy');
            self.dialog.uiDialog('destroy');

            // update/delete widget properties
            delete self.dialog;

            // reset default aspect ratio
            self.currentAspectRatio = o.defaultAspectRatio;
        },

        /**
         * Method builds crop data object.
         *
         * @member jQuery.ui.presentationUiImageCrop
         *
         * @since 7.39.0
         */
        _getCropData: function() {
            var self = this,
                o = self.options,
                cropData = self.tmplElements.image.cropper('getData'),
                imageData = self.tmplElements.image.cropper('getImageData');

            // additional image properties
            cropData.id = o.id;
            cropData.naturalHeight = imageData.naturalHeight;
            cropData.naturalWidth = imageData.naturalWidth;
            cropData.name = o.img.data('name');
            cropData.new = o.img.data('new') !== undefined ? o.img.data('new') : false;
            cropData.variation = o.variation;

            return cropData;
        },

        /**
         * Method initializes the cropperjs plugin.
         *
         * @member jQuery.ui.presentationUiImageCrop
         *
         * @since 7.39.0
         */
        _initImageCrop: function() {
            var self = this,
                o = self.options,
                cropData;

            // initialize cropper on image (self.tmplElements.image.cropper(...))
            self.tmplElements.image.cropper({
              aspectRatio: self.aspectRatioMap[o.defaultAspectRatio] ? self.aspectRatioMap[o.defaultAspectRatio].args[0] : self.aspectRatioMap['3:2'].args[0],
              // restrict the crop box to not exceed the size of the image canvas
              viewMode: 1,
              // use dragging mode (no new crop box will be created upon dragging the canvas)
              dragMode: 'move',
              toggleDragModeOnDblclick: false,
              // turn off unnecessary checks
              responsive: false,
              restore: false,
              checkCrossOrigin: false,
              // disable the dashed lines above the crop box
              guides: false,
              // enable crop on initialization
              autoCrop: true,
              // set wheel zoom ratio to have the same value as used with zoom buttons
              wheelZoomRatio: o.zoomRatio,
              // handle crop event
              crop: function(evt) {
                self._handleCrop(evt);
              }
            });

            // set input values asynchronous
            window.setTimeout(function() {
                self._setCropDimensionInputs();
            }, 100);
        },


        /**
         * Method handles crop event of cropper.
         *
         * @member jQuery.ui.presentationUiImageCrop
         *
         * @since 7.39.0
         */
        _handleCrop: function(evt) {
            var self = this;

            // set new values in input fields for width and height
            self.tmplElements.cropWidth.val(Math.round(evt.detail.width));
            self.tmplElements.cropHeight.val(Math.round(evt.detail.height));
        },


        /**
         * Method to set values for crop dimension input fields.
         *
         * @member jQuery.ui.presentationUiImageCrop
         *
         * @since 7.39.0
         */
        _setCropDimensionInputs: function() {
            var self = this,
                cropData;

            //
            cropData = self.tmplElements.image.cropper('getData');

            self.tmplElements.cropWidth.val(Math.round(cropData.width));
            self.tmplElements.cropHeight.val(Math.round(cropData.height));
        },

        /**
         * Method handles the button to disable cropping.
         *
         * @member jQuery.ui.presentationUiImageCrop
         *
         * @since 7.39.0
         */
        _handleDisableCrop: function(options) {
            var self = this;

            // disable custom aspect ratio group
            self._disableAspectRatioCustom();

            // set active button
            self.tmplElements.aspectRatio.removeClass(self.buttonActiveClass);
            self.tmplElements.disableCrop.addClass(self.buttonActiveClass);

            self._handleButton(options);
        },

        /**
         * Method handles click event of ratio button.
         *
         * @member jQuery.ui.presentationUiImageCrop
         *
         * @since 7.39.0
         */
        _handleAspectRatioButton: function(options) {
            var self = this;

            self._disableAspectRatioCustom();

            // set active button
            self.tmplElements.aspectRatio.removeClass(self.buttonActiveClass);
            self.tmplElements.disableCrop.removeClass(self.buttonActiveClass);
            options.button.addClass(self.buttonActiveClass);

            // enable cropping
            self.tmplElements.image.cropper('crop');

            // set current aspect ratio
            self.currentAspectRatio = options.aspectRatioCurrent;
            // execute aspect ratio change
            self._handleButton(options);
        },

        /**
         * Method handles click event of custom aspect ratio button.
         *
         * @member jQuery.ui.presentationUiImageCrop
         *
         * @since 7.39.0
         */
        _enableAspectRatioCustom: function(options) {
            var self = this

            self.aspectRatioCustom = true;

            // enable cropping
            self.tmplElements.image.cropper('crop');

            self.tmplElements.aspectRatioGroup.find('input').prop('disabled', false);

            // set active button
            self.tmplElements.aspectRatio.removeClass(self.buttonActiveClass);
            self.tmplElements.disableCrop.removeClass(self.buttonActiveClass);
            self.tmplElements.aspectRatioCustom.addClass(self.buttonActiveClass);

            // remove disable class (opacity) from custom group
            self.tmplElements.aspectRatioGroup.removeClass(self.customAspectRatioDiabledClass);
        },

        /**
         * Method handles click event of custom aspect ratio button.
         *
         * @member jQuery.ui.presentationUiImageCrop
         *
         * @since 7.39.0
         */
        _disableAspectRatioCustom: function() {
            var self = this;

            // handle aspect ratio lock
            self.aspectRatioLock = false;
            self._handleAspectRatioLock();

            // hande custom ratio group
            self.aspectRatioCustom = false;
            self.tmplElements.aspectRatioGroup.find('input').prop('disabled', true);
            self.tmplElements.aspectRatioCustom.removeClass(self.buttonActiveClass);
            self.tmplElements.aspectRatioGroup.addClass(self.customAspectRatioDiabledClass);
        },

        /**
         * Method handles click event of aspect ratio lock/unlock button.
         *
         * @member jQuery.ui.presentationUiImageCrop
         *
         * @since 7.39.0
         */
        _handleAspectRatioLock: function() {
            var self = this,
                classObj = {
                    true: 'fa-lock',
                    false: 'fa-unlock-alt'
                },
                cropData = self.tmplElements.image.cropper('getData'),
                // new crop data for cropper widget
                setData = {
                    x: cropData.x,
                    y: cropData.y,
                    width: getSanitizedCropSizeValue(self.tmplElements.cropWidth.val()),
                    height: getSanitizedCropSizeValue(self.tmplElements.cropHeight.val())
                },
                // new ratio for cropper widget
                ratio = (setData.width === 0 || setData.height === 0) ? self.aspectRatioMap[self.currentAspectRatio].args[0] : (setData.width/setData.height);

            // toggle custom lock state
            self.aspectRatioLock = !self.aspectRatioLock;

            // set aspect ratio depending on locked/unlocked state of the cropper
            self.tmplElements.image.cropper('setAspectRatio', self.aspectRatioLock ? ratio : 'free');
            self.tmplElements.image.cropper('setData', setData);

            self.tmplElements.aspectRatioLock
                .removeClass(classObj[!self.aspectRatioLock])
                .addClass(classObj[self.aspectRatioLock]);
        },

        /**
         * Method handles change/input events of the crop width input field.
         *
         * @member jQuery.ui.presentationUiImageCrop
         *
         * @since 7.39.0
         */
        _handleCropWidth: function() {
            var self = this,
                cropper = self.tmplElements.image.data('cropper'),
                newData = {
                    width: getSanitizedCropSizeValue(self.tmplElements.cropWidth.val())
                },
                tmpData;

            if (self.aspectRatioLock) {
              tmpData = cropper.getImageData();
              newData.height = Math.round(newData.width / self.aspectRatioMap[self.currentAspectRatio].args[0]);
            }

            cropper.setData(newData);
            self.tmplElements.cropHeight.val(Math.round(cropper.getData().height));
        },

        /**
         * Method handles change/input events of the crop height input field.
         *
         * @member jQuery.ui.presentationUiImageCrop
         *
         * @since 7.39.0
         */
        _handleCropHeight: function() {
            var self = this,
                cropper = self.tmplElements.image.data('cropper'),
                newData = {
                    height: getSanitizedCropSizeValue(self.tmplElements.cropHeight.val())
                },
                tmpData;

            if (self.aspectRatioLock) {
              tmpData = cropper.getImageData();
              newData.width = Math.round(newData.height * self.aspectRatioMap[self.currentAspectRatio].args[0]);
            }

            cropper.setData(newData);
            self.tmplElements.cropWidth.val(Math.round(cropper.getData().width));
        },

        /**
         * Method handles click event of tool bar button. Buttons defined in
         * this.navigationButtonMap.
         *
         * @member jQuery.ui.presentationUiImageCrop
         *
         * @since 7.39.0
         */
        _handleButton: function(options) {
            var self = this,
                cropper = self.tmplElements.image.data('cropper');

            // execute method
            cropper[options.method].apply(cropper, options.args);
        },

        /**
         * Method to save data after image was cropped.
         *
         * @member jQuery.ui.presentationUiImageCrop
         *
         * @since 7.39.0
         */
        _handleCropData: function() {
            var self = this,
                o = self.options,
                cropData = self._getCropData();

            // if crop data should not be saved => execute callback
            if (o.onInterruptSave && typeof o.onInterruptSave === 'function') {
                o.onInterruptSave($.extend({}, {
                    cropData: cropData
                }));
                self._resetUI();

            // save crop data
            } else {
                ep.ajax({
                    type: "POST",
                    dataType: "json",
                    url: '?',
                    data: {
                        ViewAction: "JSONViewResponse",
                        ChangeAction: "JSONSaveCropData",
                        cropData: JSON.stringify(cropData),
                        ChangeObjectID: o.objectID
                    }
                })
                .done(function(data, textStatus, jqXHR) {
                    if (data.success && data.success === true) {
                        self._resetUI();
                    } else {
                        // destroy widgets and reset data
                        self._destroyData();

                        // if error object exists
                        if (data.error) {
                            console.error('Presentation/Data/Public/lib/de_epages/presentation/ui/imagecrop.js: ', data.error);
                        }
                    }
                })
                .fail(function(jqXHR, textStatus, errorThrown) {
                    // uncomment the following line for debugging if an error occurs
                    // console.log('Presentation/Data/Public/lib/de_epages/presentation/ui/imagecrop.js (fail): ', arguments);
                });
            }

        },

        /**
         * When the method is called it checks if there is already a dialog created or if the dialog is not defined yet.
         * If not defined, the widget function _createDialog is called otherwise the dialog only opens as an overlay.
         *
         * @method open
         * @member jQuery.ui.presentationUiImageCrop
         *
         * @since 7.39.0
         */
        open: function() {
            var self = this;

            if (self.dialog) {
                self.dialog.uiDialog('open');
            } else {
                self._createDialog();
            }
        },

        /**
         * When the method is called an open dialog will be closed.
         *
         * @method close
         * @member jQuery.ui.presentationUiImageCrop
         *
         * @since 7.39.0
         */
        close: function() {
            var self = this;

            if (self.dialog) {
                self.dialog.uiDialog('close');
            }
        },

        destroy: function() {
            var self = this;

            self.tmplElements.aspectRatio.off('click.imageCrop');
            self.tmplElements.aspectRatioCustom.off('click.imageCrop');
            self.tmplElements.aspectRatioLock.off('input.imageCrop');
            self.tmplElements.cropHeight.off('click.imageCrop');
            self.tmplElements.cropWidth.off('input.imageCrop');
            self.tmplElements.disableCrop.off('click.imageCrop');
            self.tmplElements.navigationButton.off('click.imageCrop');
            self.tmplElements.rotateButton.off('click.imageCrop');

            // destroy widgets
            self.tmplElements.image && self.tmplElements.image.cropper('destroy');
            self.dialog && self.dialog.uiDialog('destroy');
            self.dialog = undefined;

            this._superApply(arguments);
        }
    });

    return de_epages;
});


